# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

from customtkinter import CTkFrame, CTkLabel


class Heading(CTkFrame):
    def __init__(self, master, text: str, *args, **kwargs) -> None:
        super().__init__(master, fg_color="gray38", *args, **kwargs)

        label = CTkLabel(
            master=self,
            text_font=("Roboto Medium", 30),  # type: ignore
            text=text,
            anchor="w",
        )
        label.grid(row=0, column=0, pady=20, padx=20, sticky="nswe")



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

# Based on https://gist.github.com/mp035/9f2027c3ef9172264532fcd6262f3b01

import platform
from tkinter import Canvas

from customtkinter import CTkFrame, CTkScrollbar


class ScrollableFrame(CTkFrame):
    def __init__(self, master, *args, **kwargs) -> None:
        super().__init__(master, *args, **kwargs)

        self.grid_rowconfigure(0, weight=1)
        self.grid_columnconfigure(0, weight=1)

        self.canvas = Canvas(
            self, borderwidth=0, background="#2A2D2E", highlightthickness=0
        )
        self.canvas.grid(row=0, column=0, sticky="nswe")

        self.content = CTkFrame(self.canvas)
        self.canvas_window = self.canvas.create_window(
            (0, 0), window=self.content, anchor="nw"
        )
        self.content.grid_columnconfigure(0, weight=1)

        self.scrollbar = CTkScrollbar(self, command=self.canvas.yview)
        self.scrollbar.grid(row=0, column=1, padx=5, pady=5, sticky="ns")
        self.canvas.configure(yscrollcommand=self.scrollbar.set)

        # bind an event whenever the size of the content frame changes.
        self.content.bind("<Configure>", self.on_frame_configure)

        # bind an event whenever the size of the canvas frame changes.
        self.canvas.bind("<Configure>", self.on_canvas_configure)

        # bind wheel events when the cursor enters the control
        self.content.bind("<Enter>", self.on_enter)

        # unbind wheel events when the cursorl leaves the control
        self.content.bind("<Leave>", self.on_leave)

        # perform an initial stretch on render, otherwise the scroll region has a tiny border until the first resize
        self.on_frame_configure(None)

    def on_frame_configure(self, event) -> None:
        """Reset the scroll region to encompass the inner frame"""

        # whenever the size of the frame changes, alter the scroll region respectively.
        self.canvas.configure(scrollregion=self.canvas.bbox("all"))

    def on_canvas_configure(self, event) -> None:
        """Reset the canvas window to encompass inner frame when required"""

        # whenever the size of the canvas changes alter the window region respectively.
        self.canvas.itemconfig(self.canvas_window, width=event.width)

    def on_mouse_wheel(self, event) -> None:
        """Cross platform scroll wheel event"""
        if platform.system() == "Windows":
            self.canvas.yview_scroll(int(-1 * (event.delta / 120)), "units")
        elif platform.system() == "Darwin":
            self.canvas.yview_scroll(int(-1 * event.delta), "units")
        else:
            if event.num == 4:
                self.canvas.yview_scroll(-1, "units")
            elif event.num == 5:
                self.canvas.yview_scroll(1, "units")

    def on_enter(self, event) -> None:
        """Bind wheel events when the cursor enters the control"""
        if platform.system() == "Linux":
            self.canvas.bind_all("<Button-4>", self.on_mouse_wheel)
            self.canvas.bind_all("<Button-5>", self.on_mouse_wheel)
        else:
            self.canvas.bind_all("<MouseWheel>", self.on_mouse_wheel)

    def on_leave(self, event) -> None:
        """Unbind wheel events when the cursor leaves the control"""
        if platform.system() == "Linux":
            self.canvas.unbind_all("<Button-4>")
            self.canvas.unbind_all("<Button-5>")
        else:
            self.canvas.unbind_all("<MouseWheel>")



# SPDX-FileCopyrightText: 2022-present Manuel Quarneti <manuelquarneti@gmail.com>
#
# SPDX-License-Identifier: GPL-3.0-only

from customtkinter import (
    BooleanVar,
    CTkButton,
    CTkEntry,
    CTkFrame,
    CTkLabel,
    CTkSwitch,
    StringVar,
)

from .components.heading import Heading
from .lib import config


class Settings(CTkFrame):
    def __init__(self, master) -> None:
        super().__init__(master=master)

        self.app_config = config.read()

        self.grid_columnconfigure(0, weight=1)

        heading = Heading(master=self, text="Settings")
        heading.grid(row=0, column=0, pady=20, padx=20, sticky="nwe")

        self.automatically_check_for_updates = BooleanVar()
        automatically_check_for_updates_frame = CTkFrame(master=self)
        automatically_check_for_updates_frame.grid(
            row=1, column=0, pady=5, padx=20, sticky="we"
        )
        automatically_check_for_updates_label = CTkLabel(
            master=automatically_check_for_updates_frame,
            text="Automatically check for updates",
            anchor="w",
        )
        automatically_check_for_updates_label.grid(
            row=0, column=0, pady=10, padx=20, sticky="w"
        )
        automatically_check_for_updates_frame.grid_columnconfigure(0, weight=1)
        automatically_check_for_updates_switch = CTkSwitch(
            master=automatically_check_for_updates_frame,
            variable=self.automatically_check_for_updates,
            text="",
        )
        automatically_check_for_updates_switch.grid(
            row=0, column=1, pady=10, padx=20, sticky="e"
        )

        self.jvm_options = StringVar()
        jvm_options_frame = CTkFrame(master=self)
        jvm_options_frame.grid(row=2, column=0, pady=5, padx=20, sticky="we")
        jvm_options_label = CTkLabel(
            master=jvm_options_frame, text="JVM options", anchor="w"
        )
        jvm_options_label.grid(row=0, column=0, pady=10, padx=20, sticky="w")
        jvm_options_frame.grid_columnconfigure(0, weight=1)
        jvm_options_entry = CTkEntry(
            master=jvm_options_frame, textvariable=self.jvm_options
        )
        jvm_options_entry.grid(row=0, column=1, pady=10, padx=20, sticky="e")

        self.jvm_memory = StringVar()
        jvm_memory_frame = CTkFrame(master=self)
        jvm_memory_frame.grid(row=3, column=0, pady=5, padx=20, sticky="we")
        jvm_memory_label = CTkLabel(
            master=jvm_memory_frame, text="JVM memory", anchor="w"
        )
        jvm_memory_label.grid(row=0, column=0, pady=10, padx=20, sticky="w")
        jvm_memory_frame.grid_columnconfigure(0, weight=1)
        jvm_memory_entry = CTkEntry(
            master=jvm_memory_frame, textvariable=self.jvm_memory
        )
        jvm_memory_entry.grid(row=0, column=1, pady=10, padx=20, sticky="e")

        # empty row as spacing
        self.grid_rowconfigure(99, weight=1)

        self.button_bar = CTkFrame(master=self)
        self.button_bar.grid(row=100, column=0, pady=0, padx=0, sticky="swe")

        # empty column as spacing
        self.button_bar.grid_columnconfigure(0, weight=1)

        self.reset_button = CTkButton(
            master=self.button_bar,
            text="Reset to default settings",
            command=self.reset_to_default_settings,
        )
        self.reset_button.grid(row=0, column=1, pady=10, padx=10, sticky="se")

        self.save_button = CTkButton(
            master=self.button_bar,
            text="Save",
            command=self.save,
        )
        self.save_button.grid(row=0, column=2, pady=10, padx=10, sticky="se")

        self.update_settings()

    def update_settings(self) -> None:
        self.automatically_check_for_updates.set(
            self.app_config["automatically_check_for_updates"]
        )
        self.jvm_options.set(" ".join(self.app_config["jvm_options"]))
        self.jvm_memory.set(self.app_config["jvm_memory"])

    def reset_to_default_settings(self) -> None:
        self.app_config = config.default()
        self.update_settings()

    def save(self) -> None:
        self.app_config[
            "automatically_check_for_updates"
        ] = self.automatically_check_for_updates.get()
        self.app_config["jvm_options"] = self.jvm_options.get().split()
        self.app_config["jvm_memory"] = self.jvm_memory.get()

        config.write(self.app_config)
